import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.concurrent.CountDownLatch;

/**
 * La sincronizzazione avviene tramite blocco synchronized.<br>
 * L'applicazione  tutta in un unico sorgente.
 */
public class AnalisiPerRigheSynchronized implements Runnable {

	static AnalisiPerRigheSynchronized[] righe;
	static Thread[] threads;
	static int somma = 0;
	static CountDownLatch barrier;

	/**
	 * uso: java AnalisiPerRigheSynchronized [immagine].
	 *
	 * @param args l'immagine da usare (default image.png).
	 *
	 * @throws Exception se si verifica un problema.
	 */
	public static void main(String[] args) throws Exception {
		File f = new File(args.length>0 ? args[0] : "image.png");
		BufferedImage image = ImageIO.read(f);
		int w = image.getWidth();
		int h = image.getHeight();
		righe = new AnalisiPerRigheSynchronized[h];
		threads = new Thread[h];
		barrier = new CountDownLatch(h);
		for(int i=0; i<h; i++) {
			righe[i] = new AnalisiPerRigheSynchronized(image.getRGB(0, i, w, 1, null, 0, w));
			threads[i] = new Thread(righe[i]);
		}
		for(int i=0; i<h; i++) {
			threads[i].start();
		}
		barrier.await();
		System.out.println("Blu medio: "+(somma/h));
	}

	private int[] rgbArray;
	private int max = -1;
	private boolean done = false;

	AnalisiPerRigheSynchronized(int[] a) {
		rgbArray = a;
	}

	private Object monitor; // usato solo per la sincronizzazione, qualunque oggetto potrebbe andare bene
	public void run() {
		for(int i=0; i<rgbArray.length; i++) {
			int blu = rgbArray[i] & 255;
			if(blu > max) max = blu;
		}
		synchronized(monitor) {
			somma += max;
		}
		done = true;
		barrier.countDown();
	}

	int getMax() {
		if(!done) throw new RuntimeException("done == false");
		return max;
	}

}
